@@ -1,4 +1,4 @@
-// $Id: issupportedstring.c,v 1.3 2004/04/13 08:04:51 ensc Exp $    --*- c -*--
+// $Id: issupportedstring.c,v 1.4 2005/05/19 18:05:18 ensc Exp $    --*- c -*--
 
 // Copyright (C) 2004 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
@@ -33,7 +33,7 @@ static struct {
 } FEATURES[] = {
   DECL(VKILL),   DECL(IATTR),     DECL(RLIMIT), DECL(COMPAT),
   DECL(MIGRATE), DECL(NAMESPACE), DECL(SCHED),  DECL(VINFO),
-  DECL(VHI),     DECL(VSHELPER0), DECL(VSHELPER)
+  DECL(VHI),     DECL(VSHELPER0), DECL(VSHELPER), DECL(VWAIT)
 };
 
 bool
