@@ -1,4 +1,4 @@
-// $Id: chcontext.c,v 1.23 2004/05/11 18:30:06 ensc Exp $
+// $Id: chcontext.c,v 1.24 2005/03/22 15:05:24 ensc Exp $
 
 // Copyright (C) 2003,2004 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 // based on chcontext.cc by Jacques Gelinas
@@ -211,7 +211,7 @@ setFlags(char const *str, uint32_t *flags)
 
   if (err.ptr!=0) {
     WRITE_MSG(2, "Unknown flag '");
-    write(2, err.ptr, err.len);
+    Vwrite(2, err.ptr, err.len);
     WRITE_MSG(2, "'\n");
     exit(wrapper_exit_code);
   }
@@ -260,7 +260,7 @@ tellContext(xid_t ctx)
   l = utilvserver_fmt_long(buf,ctx);
 
   WRITE_MSG(1, "New security context is ");
-  write(1, buf, l);
+  Vwrite(1, buf, l);
   WRITE_MSG(1, "\n");
 }
 
