@@ -1,4 +1,4 @@
-// $Id: reducecap.c,v 1.17 2005/01/27 20:38:35 ensc Exp $
+// $Id: reducecap.c,v 1.18 2005/03/24 12:44:17 ensc Exp $
 
 // Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 // based on reducecap.cc by Jacques Gelinas
@@ -187,7 +187,7 @@ printReducecap(struct __user_cap_data_struct *user)
     buf[23 + 10*0 + 4] = (user->effective   & bit) ? 'X' : ' ';
     buf[23 + 10*1 + 4] = (user->permitted   & bit) ? 'X' : ' ';
     buf[23 + 10*2 + 4] = (user->inheritable & bit) ? 'X' : ' ';
-    write(1, buf, len);
+    Vwrite(1, buf, len);
   }
 }
 
@@ -258,7 +258,7 @@ int main (int argc, char *argv[])
 	flags = vc_list2cflag_compat(optarg, 0, &err);
 	if (err.ptr!=0) {
 	  WRITE_MSG(2, "Unknown flag '");
-	  write(2, err.ptr, err.len);
+	  Vwrite(2, err.ptr, err.len);
 	  WRITE_MSG(2, "'\n");
 	  exit(wrapper_exit_code);
 	}
