@@ -1,4 +1,4 @@
-// $Id: secure-mount.c,v 1.20 2005/01/26 15:29:40 ensc Exp $    --*- c++ -*--
+// $Id: secure-mount.c,v 1.24 2005/03/24 12:45:06 ensc Exp $    --*- c++ -*--
 
 // Copyright (C) 2003 Enrico Scholz <enrico.scholz@informatik.tu-chemnitz.de>
 //  
@@ -54,6 +54,7 @@
 #include <signal.h>
 
 #define ENSC_WRAPPERS_FCNTL	1
+#define ENSC_WRAPPERS_UNISTD	1
 #include <wrappers.h>
 
 #define MNTPOINT	"/etc"
@@ -66,6 +67,7 @@ struct MountInfo {
     char const *	type;
     unsigned long	flag;
     unsigned long	xflag;
+    unsigned long	mask;
     char *		data;
 };
 
@@ -316,11 +318,20 @@ callExternalMount(struct MountInfo const *mnt)
   if      (mnt->flag & MS_BIND) argv[idx++] = "--bind";
   else if (mnt->flag & MS_MOVE) argv[idx++] = "--move";
 
-  if (mnt->data &&
+  argv[idx++] = "-o";
+  if (mnt->data && *mnt->data &&
       strcmp(mnt->data, "defaults")!=0) {
-    argv[idx++] = "-o";
-    argv[idx++] = mnt->data;
+    if (mnt->mask & MS_NODEV)
+      argv[idx++] = mnt->data;
+    else {
+      char *	tmp = alloca(strlen(mnt->data) + sizeof("nodev,"));
+      strcpy(tmp, "nodev,");
+      strcat(tmp, mnt->data);
+      argv[idx++] = tmp;
+    }
   }
+  else
+    argv[idx++] = "nodev";
 
   if (mnt->type) {
     argv[idx++] = "-t";
@@ -396,9 +407,12 @@ mountSingle(struct MountInfo const *mnt, struct Options const *opt)
     return false;
 
   if (mnt->flag & (MS_BIND|MS_MOVE)) {
+    unsigned long	flag = mnt->flag;
+    if ((flag & MS_NODEV)==0) flag |= MS_NODEV;
+    
     if (mount(mnt->src, ".",
 	      mnt->type ? mnt->type : "",
-	      mnt->flag,  mnt->data)==-1) {
+	      flag,  mnt->data)==-1) {
       perror("secure-mount: mount()");
       return false;
     }
@@ -440,6 +454,7 @@ transformOptionList(struct MountInfo *info, size_t UNUSED *col)
     if (opt!=0) {
       info->flag  &= ~opt->mask;
       info->flag  |=  opt->flag;
+      info->mask  |=  opt->mask;
       info->xflag |=  opt->xflag;
     }
 
@@ -479,11 +494,14 @@ static enum {prDOIT, prFAIL, prIGNORE}
   info->data = buf;
   MOVE_TO_NEXT_FIELD(buf, true);
 
-  if (strcmp(info->type, "swap")==0) return prIGNORE;
-  if (strcmp(info->type, "none")==0) info->type = 0;
+  info->flag  = MS_NODEV;
+  info->mask  = 0;
+  info->xflag = 0;
+
+  if      (strcmp(info->type, "swap")  ==0) return prIGNORE;
+  else if (strcmp(info->type, "none")  ==0) info->type  = 0;
+  else if (strcmp(info->type, "devpts")==0) info->mask |= MS_NODEV;
 
-  info->flag    = 0;
-  info->xflag   = 0;
   if (col) *col = err_col;
   if (!transformOptionList(info,col)) return prFAIL;
   if (info->xflag & XFLAG_NOAUTO)     return prIGNORE;
@@ -503,7 +521,7 @@ showFstabPosition(int fd, char const *fname, size_t line_nr, size_t col_nr)
   buf[len++] = ':';
   len += utilvserver_fmt_uint(buf+len, col_nr);
   WRITE_STR(fd, fname);
-  write(fd, buf, len);
+  Vwrite(fd, buf, len);
 }
 
 
@@ -555,7 +573,7 @@ mountFstab(struct Options const *opt)
 	case prIGNORE	:  break;
 	case prDOIT	: {
 	  bool		is_rootfs = (strcmp(mnt.dst, "/")==0);
-	  chdir("/");
+	  Echdir("/");
 	  if (( is_rootfs && opt->rootfs==rfsNO) ||
 	      (!is_rootfs && opt->rootfs==rfsONLY)) { /* ignore the entry */ }
 	  else if (!mountSingle(&mnt, opt)) {
